import {marked} from "marked";
import {TableFactory} from "./factory";
// Set options
// `highlight` example uses https://highlightjs.org
marked.setOptions({
    renderer: new marked.Renderer(),
    highlight: function(code, lang) {
        console.log('code',code,'lang',lang)
        const hljs = require('highlight.js');
        const language = hljs.getLanguage(lang) ? lang : 'plaintext';
        return hljs.highlight(code, { language }).value;
    },
    langPrefix: 'hljs language-', // highlight.js css expects a top-level 'hljs' class.
    pedantic: false,
    gfm: true,
    breaks: false,
    sanitize: false,
    smartLists: true,
    smartypants: false,
    xhtml: false,
    headerIds: true
});

// Override function

export class MarkDown {
    constructor(markText) {
        this.markText = markText;
        this.heads = [];
        this.cache = {};
        this.tables = []
        this.$this = this;
        this.tableFactory = new TableFactory({$this:this})
    }
}

MarkDown.prototype.getMarkHtml = function () {
    let extension = {};
    if (this.tokenizer) {
        extension["tokenizer"] = this.tokenizer;
    }
    if (this.renderer) {
        extension["renderer"] = this.renderer;
    }
    if (extension) {
        extension["markdownFactory"] = this.$this;
        marked.use({...extension});
    }

    return marked.parse(this.markText);
};

MarkDown.prototype.getHeads = function () {
    return this.heads;
};
MarkDown.prototype.getTables = function () {
    return [...this.tables];
};

MarkDown.prototype.tokenizer = {};

MarkDown.prototype.renderer = {
    // Override function
    heading(text, level, raw, slugger) {
        const categorize = {
            type: "heading",
            raw: raw,
            depth: level,
            text
        };
        let $this = this.options.markdownFactory;
        if ($this && $this.heads) {
            $this.heads.push(categorize);
        }

        const escapedText = text.toLowerCase().replace(/[^\w]+/g, "-");

        categorize["html"] = `
            <h${level}>
              <a name="${escapedText}" class="anchor" href="#${escapedText}">
                <span class="header-link"></span>
              </a>
              ${text}
            </h${level}>`;

        if (this.options.headerIds) {
            const id = this.options.headerPrefix + slugger.slug(raw);
            categorize["id"] = id;
            return `<h${level} id="${id}">${text}</h${level}>\n`;
        }
        return categorize.html;
    },


    tablerow(content, options) {
        let $this = this.options.markdownFactory;
        let tableFactory = $this.tableFactory
        tableFactory.pushRow()
        return `<tr>\n${content}</tr>\n`;
    },

    tablecell(content, flags) {
        let $this = this.options.markdownFactory;
        let tableFactory = $this.tableFactory
        const type = flags.header ? 'th' : 'td';
        if (flags.header) {
            tableFactory.setTableColumn(content)
        } else {
            tableFactory.setTableRow(content)
        }
        //default tablecell
        const tag = flags.align
            ? `<${type} align="${flags.align}">`
            : `<${type}>`;
        return tag + content + `</${type}>\n`;
    },

    table(header, body, options) {
        let $this = this.options.markdownFactory;
        let tableFactory = $this.tableFactory
        let html = tableFactory.getTableHtml()
        if (html) {
            return '\n'+html+'\n';
            // return '\n'+'<el-table :data="Tablel60822r1io1b3da5ymbData" style="width: 100%"><el-table-column prop="1hEhgo3WgL9pANrhNa5WD3isVdPH" label="Foo" ></el-table-column> <el-table-column prop="ByVFa7xVHsyW6CuvFH1hHy" label="Bar" ></el-table-column> <el-table-column prop="GT65fHJgu7cGBVgwKVBe3uTtiDL9WF" label="Baz" ></el-table-column></el-table>'+'\n';
            // return ' <ve-table :columns="columns" :table-data="tableData" />\n'
        } else {
            if (body) body = `<tbody>${body}</tbody>`;

            return '<table>\n'
                + '<thead>\n'
                + header
                + '</thead>\n'
                + body
                + '</table>\n';
        }
    }
};
